/*
 * Copyright (c) 2009 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA Corporation is strictly prohibited.
 */

#ifndef __KD_NV_display_h
#define __KD_NV_display_h

#include <KD/kd.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Note: OpenKODE non-I/O indices 208-463 have been reserved by NVIDIA for
         for this extension. This turns out to be far more than currently
         needed. Some will be set aside for additional display system
         properties which may be added to the extension, others will be used
         for the window manager extension, and the rest will be returned to
         the OpenKODE registry. */


/* Display/desktop types */
typedef struct KDDisplaySystemNV KDDisplaySystemNV;
typedef struct KDDesktopNV KDDesktopNV;
typedef struct KDDisplayNV KDDisplayNV;
typedef struct KDAdapterNV KDAdapterNV;
typedef struct KDDisplayModeNV {
    KDuint width;
    KDuint height;
    KDuint refresh;
} KDDisplayModeNV;

/* Display system functions */
KD_API KDint KD_APIENTRY kdGetNativeDisplayNV(const KDchar* desktopname, KDint desktoplayer, const KDchar* displayname, EGLNativeDisplayType* native);
KD_API KDDisplaySystemNV* KD_APIENTRY kdCreateDisplaySystemSnapshotNV(void* eventuserptr);
KD_API KDint KD_APIENTRY kdUpdateDisplaySystemSnapshotNV(KDDisplaySystemNV* sys);
KD_API KDint KD_APIENTRY kdDeleteDisplaySystemSnapshotNV(KDDisplaySystemNV* sys);
KD_API KDint KD_APIENTRY kdGetDisplaySystemPropertybvNV(KDDisplaySystemNV* sys, KDint pname, KDint index, KDboolean* param);
KD_API KDint KD_APIENTRY kdGetDisplaySystemPropertyivNV(KDDisplaySystemNV* sys, KDint pname, KDint index, KDint32* param);
KD_API KDint KD_APIENTRY kdGetDisplaySystemPropertycvNV(KDDisplaySystemNV* sys, KDint pname, KDint index, KDchar* param, KDsize* size);

/* Desktop functions */
KD_API KDDesktopNV* KD_APIENTRY kdGetDesktopNV(const KDchar* name, void* eventuserptr);
KD_API KDint KD_APIENTRY kdReleaseDesktopNV(KDDesktopNV* desktop);
KD_API KDint KD_APIENTRY kdGetDesktopPropertybvNV(KDDesktopNV* desktop, KDint pname, KDboolean* param);
KD_API KDint KD_APIENTRY kdGetDesktopPropertyivNV(KDDesktopNV* desktop, KDint pname, KDint32* param);
KD_API KDint KD_APIENTRY kdGetDesktopPropertycvNV(KDDesktopNV* desktop, KDint pname, KDchar* param, KDsize* size);
KD_API KDint KD_APIENTRY kdSetDesktopPropertybvNV(KDDesktopNV* desktop, KDint pname, const KDboolean* param);
KD_API KDint KD_APIENTRY kdSetDesktopPropertyivNV(KDDesktopNV* desktop, KDint pname, const KDint32* param);
KD_API KDint KD_APIENTRY kdSetDesktopPropertycvNV(KDDesktopNV* desktop, KDint pname, const KDchar* param);

KD_API KDint KD_APIENTRY kdGetDesktopLayerPropertyivNV(KDDesktopNV* desktop, KDint layer, KDint pname, KDint32* param);
KD_API KDint KD_APIENTRY kdGetDesktopLayerPropertyfvNV(KDDesktopNV* desktop, KDint layer, KDint pname, KDfloat32* param);
KD_API KDint KD_APIENTRY kdSetDesktopLayerPropertyivNV(KDDesktopNV* desktop, KDint layer, KDint pname, const KDint32* param);
KD_API KDint KD_APIENTRY kdSetDesktopLayerPropertyfvNV(KDDesktopNV* desktop, KDint layer, KDint pname, const KDfloat32* param);

/* Display functions */
KD_API KDDisplayNV* KD_APIENTRY kdGetDisplayNV(const KDchar* name, void *eventuserptr);
KD_API KDint KD_APIENTRY kdReleaseDisplayNV(KDDisplayNV* display);
KD_API KDint KD_APIENTRY kdGetDisplayPropertybvNV(KDDisplayNV* display, KDint pname, KDboolean* param);
KD_API KDint KD_APIENTRY kdGetDisplayPropertyivNV(KDDisplayNV* display, KDint pname, KDint32* param);
KD_API KDint KD_APIENTRY kdGetDisplayPropertycvNV(KDDisplayNV* display, KDint pname, KDchar* param, KDsize* size);
KD_API KDint KD_APIENTRY kdSetDisplayPropertybvNV(KDDisplayNV* display, KDint pname, const KDboolean* param);
KD_API KDint KD_APIENTRY kdSetDisplayPropertyivNV(KDDisplayNV* display, KDint pname, const KDint32* param);
KD_API KDint KD_APIENTRY kdSetDisplayPropertycvNV(KDDisplayNV* display, KDint pname, const KDchar* param);
KD_API KDint KD_APIENTRY kdGetDisplayModeNV(KDDisplayNV* display, KDDisplayModeNV* mode);
KD_API KDint KD_APIENTRY kdSetDisplayModeNV(KDDisplayNV* display, const KDDisplayModeNV* mode, KDint protocol);
KD_API KDint KD_APIENTRY kdSearchDisplayModesNV(KDDisplayNV* display, const KDDisplayModeNV* minmode, const KDDisplayModeNV* maxmode, KDDisplayModeNV* modelist, KDsize* size);

/* Adapter functions */
KD_API KDAdapterNV* KD_APIENTRY kdGetAdapterNV(const KDchar* name, void *eventuserptr);
KD_API KDint KD_APIENTRY kdReleaseAdapterNV(KDAdapterNV* adapter);
KD_API KDint KD_APIENTRY kdGetAdapterPropertybvNV(KDAdapterNV* adapter, KDint pname, KDboolean* param);
KD_API KDint KD_APIENTRY kdGetAdapterPropertyivNV(KDAdapterNV* adapter, KDint pname, KDint32* param);
KD_API KDint KD_APIENTRY kdGetAdapterPropertycvNV(KDAdapterNV* adapter, KDint pname, KDchar* param, KDsize* size);
KD_API KDint KD_APIENTRY kdSetAdapterPropertybvNV(KDAdapterNV* adapter, KDint pname, const KDboolean* param);
KD_API KDint KD_APIENTRY kdSetAdapterPropertyivNV(KDAdapterNV* adapter, KDint pname, const KDint32* param);
KD_API KDint KD_APIENTRY kdSetAdapterPropertycvNV(KDAdapterNV* adapter, KDint pname, const KDchar* param);

/* Default display system targets */
#define KD_DEFAULT_DESKTOP_NV "kddefaultdesktopnv"
#define KD_DEFAULT_DISPLAY_NV "kddefaultdisplaynv"
#define KD_DEFAULT_ADAPTER_NV "kddefaultadapternv"


/* Display system properties. */
/* Using values 208-234. Values 235-249 reserved for future capabilities. */
#define KD_DISPLAYCAPABILITY_SNAPSHOT_FIXED_NV           208
#define KD_DISPLAYCAPABILITY_SNAPSHOT_EVENT_NV           210
#define KD_DISPLAYCAPABILITY_DESKTOP_SIZE_FIXED_NV       211
#define KD_DISPLAYCAPABILITY_DESKTOP_SIZE_ALLOW_NV       212
#define KD_DISPLAYCAPABILITY_DESKTOP_SIZE_EVENT_NV       213
#define KD_DISPLAYCAPABILITY_DESKTOP_BITS_FIXED_NV       214
#define KD_DISPLAYCAPABILITY_DESKTOP_BITS_ALLOW_NV       215
#define KD_DISPLAYCAPABILITY_DESKTOP_BITS_EVENT_NV       216
#define KD_DISPLAYCAPABILITY_DISPLAY_DETECT_FIXED_NV     217
#define KD_DISPLAYCAPABILITY_DISPLAY_DETECT_ALLOW_NV     218
#define KD_DISPLAYCAPABILITY_DISPLAY_DETECT_EVENT_NV     219
#define KD_DISPLAYCAPABILITY_DISPLAY_ENABLE_FIXED_NV     220
#define KD_DISPLAYCAPABILITY_DISPLAY_ENABLE_ALLOW_NV     221
#define KD_DISPLAYCAPABILITY_DISPLAY_ENABLE_EVENT_NV     222
#define KD_DISPLAYCAPABILITY_DISPLAY_MODE_FIXED_NV       223
#define KD_DISPLAYCAPABILITY_DISPLAY_MODE_ALLOW_NV       224
#define KD_DISPLAYCAPABILITY_DISPLAY_MODE_EVENT_NV       225
#define KD_DISPLAYCAPABILITY_DISPLAY_MAPPING_FIXED_NV    226
#define KD_DISPLAYCAPABILITY_DISPLAY_MAPPING_ALLOW_NV    227
#define KD_DISPLAYCAPABILITY_DISPLAY_MAPPING_EVENT_NV    228
#define KD_DISPLAYCAPABILITY_DISPLAY_APERTURE_FIXED_NV   229
#define KD_DISPLAYCAPABILITY_DISPLAY_APERTURE_ALLOW_NV   230
#define KD_DISPLAYCAPABILITY_DISPLAY_APERTURE_EVENT_NV   231
#define KD_DISPLAYCAPABILITY_DESKTOP_LAYERS_FIXED_NV     232
#define KD_DISPLAYCAPABILITY_DESKTOP_LAYERS_ALLOW_NV     233
#define KD_DISPLAYCAPABILITY_DESKTOP_LAYERS_EVENT_NV     234

/* Desktop properties */
/* Using values 250-254. Values 255-259 reserved for future properties */
#define KD_DESKTOPPROPERTY_COUNT_NV                      250
#define KD_DESKTOPPROPERTY_NAME_NV                       251
#define KD_DESKTOPPROPERTY_SIZE_NV                       252
#define KD_DESKTOPPROPERTY_BITS_NV                       253
#define KD_DESKTOPPROPERTY_LAYERS_NV                     254

/* Display properties */
/* Using values 260-275. Values 276-279 reserved for future properties */
#define KD_DISPLAYPROPERTY_COUNT_NV                      260
#define KD_DISPLAYPROPERTY_NAME_NV                       261
#define KD_DISPLAYPROPERTY_VENDOR_NV                     262
#define KD_DISPLAYPROPERTY_DEVICE_NV                     263
#define KD_DISPLAYPROPERTY_ADAPTER_NAME_NV               264
#define KD_DISPLAYPROPERTY_DESKTOP_NAME_NV               265
#define KD_DISPLAYPROPERTY_CONNECTED_NV                  266
#define KD_DISPLAYPROPERTY_ENABLED_NV                    267
#define KD_DISPLAYPROPERTY_POWER_NV                      268
#define KD_DISPLAYPROPERTY_APERTURE_NV                   269
#define KD_DISPLAYPROPERTY_MIN_APERTURE_SIZE_NV          270
#define KD_DISPLAYPROPERTY_MAX_APERTURE_SIZE_NV          271
#define KD_DISPLAYPROPERTY_HDCP_NV                       272
#define KD_DISPLAYPROPERTY_LUT_INPUT_SIZE_NV             273
#define KD_DISPLAYPROPERTY_LUT_MAX_OUTPUT_NV             274
#define KD_DISPLAYPROPERTY_LUT_RGB_NV                    275


/* Adapter properties */
/* Using values 280-283. Values 284-289 reserved for future properties */
#define KD_ADAPTERPROPERTY_COUNT_NV                      280
#define KD_ADAPTERPROPERTY_NAME_NV                       281
#define KD_ADAPTERPROPERTY_VENDOR_NV                     282
#define KD_ADAPTERPROPERTY_DEVICE_NV                     283

/* Window properties */
/* Using values 290-293. Values 294-299 reserved for future properties */
#define KD_WINDOWPROPERTY_DESKTOP_NAME_NV                290
#define KD_WINDOWPROPERTY_DESKTOP_LAYER_NV               291
#define KD_WINDOWPROPERTY_DESKTOP_OFFSET_NV              292
#define KD_WINDOWPROPERTY_DESKTOP_EXCLUSIVE_NV           293

/* Events */
/* Using values 300-304. Values 305-309 reserved for future events */
#define KD_EVENT_DISPLAYSYSTEM_CHANGE_NV                 300
#define KD_EVENT_ADAPTERPROPERTY_CHANGE_NV               301
#define KD_EVENT_DESKTOPPROPERTY_CHANGE_NV               302
#define KD_EVENT_DISPLAYPROPERTY_CHANGE_NV               303
#define KD_EVENT_DISPLAYMODE_CHANGE_NV                   304

/* Display power settings */
/* Using values 310-313. None reserved */
#define KD_DISPLAY_POWER_ON                              310
#define KD_DISPLAY_POWER_STANDBY                         311
#define KD_DISPLAY_POWER_SUSPEND                         312
#define KD_DISPLAY_POWER_OFF                             313

/* Display protocols */
/* Using values 314-338. Values 339-349 reserved for future protocols */
#define KD_DISPLAY_PROTOCOL_AUTOMATIC_NV                 314
#define KD_DISPLAY_PROTOCOL_VGA_NV                       315
#define KD_DISPLAY_PROTOCOL_DVIA_NV                      316
#define KD_DISPLAY_PROTOCOL_DVID_NV                      317
#define KD_DISPLAY_PROTOCOL_NTSC_NV                      318
#define KD_DISPLAY_PROTOCOL_NTSC_J_NV                    319
#define KD_DISPLAY_PROTOCOL_NTSC_M_NV                    320
#define KD_DISPLAY_PROTOCOL_PAL_NV                       321
#define KD_DISPLAY_PROTOCOL_PAL_B_NV                     322
#define KD_DISPLAY_PROTOCOL_PAL_D_NV                     323
#define KD_DISPLAY_PROTOCOL_PAL_G_NV                     324
#define KD_DISPLAY_PROTOCOL_PAL_H_NV                     325
#define KD_DISPLAY_PROTOCOL_PAL_I_NV                     326
#define KD_DISPLAY_PROTOCOL_PAL_K_NV                     327
#define KD_DISPLAY_PROTOCOL_PAL_M_NV                     328
#define KD_DISPLAY_PROTOCOL_PAL_N_NV                     329
#define KD_DISPLAY_PROTOCOL_PAL_Nc_NV                    330
#define KD_DISPLAY_PROTOCOL_480P_NV                      331
#define KD_DISPLAY_PROTOCOL_480I_NV                      332
#define KD_DISPLAY_PROTOCOL_576P_NV                      333
#define KD_DISPLAY_PROTOCOL_576I_NV                      334
#define KD_DISPLAY_PROTOCOL_720P_NV                      335
#define KD_DISPLAY_PROTOCOL_720I_NV                      336
#define KD_DISPLAY_PROTOCOL_1080P_NV                     337
#define KD_DISPLAY_PROTOCOL_1080I_NV                     338

/* HDCP settings */
/* Using values 350-352. None reserved */
#define KD_DISPLAY_COPY_FREE_NV                          350
#define KD_DISPLAY_COPY_ONCE_NV                          351
#define KD_DISPLAY_COPY_NEVER_NV                         352

/* Desktop layer properties */
/* Using values 353-354. 355-365 reserved */
#define KD_DESKTOPPROPERTY_LAYER_BLENDMODE_NV            353
#define KD_DESKTOPPROPERTY_LAYER_ALPHA_NV                354

/* Constants to be used for blend mode */
#define KD_DESKTOP_LAYER_BLENDMODE_ONE_NV                 0
#define KD_DESKTOP_LAYER_BLENDMODE_CONSTANT_ALPHA_NV      1
#define KD_DESKTOP_LAYER_BLENDMODE_SRC_ALPHA_NV           2
#define KD_DESKTOP_LAYER_BLENDMODE_PREMULT_SRC_ALPHA_NV   3
#define KD_DESKTOP_LAYER_BLENDMODE_DST_ALPHA_NV           4
#define KD_DESKTOP_LAYER_BLENDMODE_PREMULT_DST_ALPHA_NV   5


#ifdef __cplusplus
}
#endif

#endif /* __KD_NV_display_h_ */
