/* Copyright (c) Mark J. Kilgard, 1997. */

/* This program is freely distributable without licensing fees  and is
   provided without guarantee or warrantee expressed or  implied. This
   program is -not- in the public domain. */

#ifndef __NV_TEXFONT_H__
#define __NV_TEXFONT_H__

#ifdef __cplusplus
extern "C" {
#endif

#define TXF_FORMAT_BYTE     0
#define TXF_FORMAT_BITMAP   1

typedef void * NVTexfont2Context;

typedef enum {
    NV_TEXFONT_HELVETICA = 0,
    NV_TEXFONT_DEFAULT = 0,
} NVTexfont2RasterFontName;

typedef enum {
    NV_VECTOR_TEXFONT_DEFAULT = 0,
    NV_VECTOR_TEXFONT_INCONSOLATA = 0
} NVTexfont2VectorFontName;

typedef struct _NVTexfont2RasterFont NVTexfont2RasterFont;
typedef struct _NVTexfont2VectorFont NVTexfont2VectorFont;

/* General Calls */

extern char     *nvtexfont2ErrorString(void);

extern NVTexfont2Context nvtexfont2AllocContext(void);
extern void              nvtexfont2FreeContext(NVTexfont2Context tfc);

extern void      nvtexfont2SetContextPos(NVTexfont2Context tfc, float x, float y);
extern void      nvtexfont2SetContextScale(NVTexfont2Context tfc, float scale);
extern void      nvtexfont2SetContextColor(NVTexfont2Context tfc, float r, float g, float b);


/* Raster Font Calls */

extern int       nvtexfont2InRasterFont(NVTexfont2RasterFont * txf,
                                       int c);
extern NVTexfont2RasterFont *nvtexfont2InitRasterFont(NVTexfont2RasterFontName font,
                                                GLuint        texobj,
                                                GLboolean     setupMipmaps,
                                                GLenum        minFilter,
                                                GLenum        magFilter);

extern void      nvtexfont2UnloadRasterFont(NVTexfont2RasterFont * txf);

extern void      nvtexfont2RenderString_All(NVTexfont2RasterFont * txf,
                                            char *string,
                                            float x, float y,
                                            float scale,
                                            float r, float g, float b);
extern void      nvtexfont2RenderString_Pos(NVTexfont2RasterFont * txf, NVTexfont2Context tfc,
                                            char *string,
                                            float x, float y);
extern void      nvtexfont2RenderString(NVTexfont2RasterFont * txf, NVTexfont2Context tfc,
                                        char *string);

extern void      nvtexfont2GetStringMetrics(NVTexfont2RasterFont * txf,
                                            char *string,
                                            int len,
                                            int *width,
                                            int *max_ascent,
                                            int *max_descent);

/* Vector Font Calls */

extern int       nvtexfont2InVectorFont(NVTexfont2VectorFont * vtf,
                                        int c);

extern NVTexfont2VectorFont *nvtexfont2InitVectorFont(NVTexfont2VectorFontName font,
                                                      GLboolean     antialias,
                                                      GLboolean     use_vbo);

extern void      nvtexfont2UnloadVectorFont(NVTexfont2VectorFont * vtf);

extern void      nvtexfont2RenderVecString_All(NVTexfont2VectorFont * vtf,
                                               char *string,
                                               float x, float y,
                                               float scale,
                                               float r, float g, float b);
extern void      nvtexfont2RenderVecString_Pos(NVTexfont2VectorFont * vtf, NVTexfont2Context tfc,
                                               char *string,
                                               float x, float y);
extern void      nvtexfont2RenderVecString(NVTexfont2VectorFont * vtf, NVTexfont2Context tfc,
                                           char *string);





#ifdef __cplusplus
}
#endif

#endif /* __NV_TEXFONT_H__ */

