NV_TOPDIR  = $(HOME)/nvsamples
NV_PLATFORM_DIR = $(NV_TOPDIR)
NV_KERNDIR = $(NV_TOPDIR)/../kernel

NV_PLATFORM_OS     = Linux
NV_PLATFORM_CPU    = armv7

NV_HOST_OS = $(shell uname)
ifeq ($(NV_HOST_OS),Linux)
   # Linux environment
   NV_HOST_OSTYPE = linux
else
   # Windows environment
   NV_HOST_OSTYPE = win32
endif

# This assumes that we strip the object file in the end.
# Adding -g and stripping gives the same binary image as the one
# built without -g.
# This gives us a debug build before stripping for better debugging.
NV_PLATFORM_CFLAGS    = -g                                        \
                        -mtune=cortex-a9                          \
                        -march=armv7-a                            \
                        -mhard-float                              \
                        -mthumb-interwork                         \
                        -mfloat-abi=softfp                        \
                        -mfpu=vfpv3-d16                           \
                        -fno-strict-aliasing                      \
                        -finline-functions                        \
                        -finline-limit=300                        \
                        -fomit-frame-pointer                      \
                        -fgcse-after-reload                       \
                        -Wall                                     \
                        -Wcast-align                              \
                        -fPIC
NV_PLATFORM_CPPFLAGS  = -DNV_GLES_VER_MAJOR=2 -DVIBRANTE

# Detects whether X11 is supported. Defaults to KD.
ifneq ($(wildcard $(NV_PLATFORM_DIR)/include/X11),)
   NV_PLATFORM_CPPFLAGS += -DX11
else
   NV_PLATFORM_CPPFLAGS += -DKD
endif

NV_PLATFORM_LDFLAGS   =

NV_PLATFORM_SDK_INC_DIR = $(NV_TOPDIR)/include
NV_PLATFORM_SDK_LIB_DIR = $(NV_TOPDIR)/lib-target
ROOTFS_LIB_DIR          = $(NV_TOPDIR)/../T20_LinuxImageV1.0Beta4/rootfs/usr/lib

NV_PLATFORM_SDK_INC   = -I$(NV_PLATFORM_SDK_INC_DIR)
NV_PLATFORM_SDK_LIB   = -L$(NV_PLATFORM_SDK_LIB_DIR) \
                        -L$(ROOTFS_LIB_DIR) \
                        -Xlinker -rpath-link \
			-Xlinker $(ROOTFS_LIB_DIR)
NV_PLATFORM_OPT       = -Os -O2
NV_PLATFORM_MATHLIB   = -lm
NV_PLATFORM_THREADLIB = -lpthread

# separately installed open source packages

NV_PLATFORM_OPENMAX_IL_INC       = -I$(NV_TOPDIR)/vibrante-e1162/include/OpenMAX/il
NV_PLATFORM_OPENMAX_ILCLIENT_INC = -I$(NV_TOPDIR)/vibrante-e1162/include/OpenMAX/ilclient

NV_PLATFORM_ALSA_DIR = $(NV_TOPDIR)/vibrante-e1162/oss/alsa/libasound2
NV_PLATFORM_ALSA_INC = -I$(NV_PLATFORM_ALSA_DIR)/usr/include
NV_PLATFORM_ALSA_LIB = -L$(NV_PLATFORM_ALSA_DIR)/usr/lib

NV_PLATFORM_JPEG_DIR = $(NV_TOPDIR)/vibrante-e1162/oss/jpeg/jpeg-lib
NV_PLATFORM_JPEG_INC = -I$(NV_PLATFORM_JPEG_DIR)/usr/include
NV_PLATFORM_JPEG_LIB = -L$(NV_PLATFORM_JPEG_DIR)/usr/lib

NV_PLATFORM_GLIB_DIR      = $(NV_TOPDIR)/vibrante-e1162/oss/glib/libglib2.0
NV_PLATFORM_GSTREAMER_DIR = $(NV_TOPDIR)/vibrante-e1162/oss/gstreamer/libgstreamer0.10
NV_PLATFORM_LIBXML2_DIR   = $(NV_TOPDIR)/vibrante-e1162/oss/xml2/libxml2
NV_PLATFORM_LIBGST_PLUGINS_BASE_DIR = $(NV_TOPDIR)/vibrante-e1162/oss/gstreamer/libgstreamer-plugins-base0.10

NV_PLATFORM_PCRE_DIR = $(NV_PLATFORM_DIR)/oss/pcre/libpcre3
NV_PLATFORM_ZLIB_DIR = $(NV_PLATFORM_DIR)/oss/zlib/zlib1g

# compiler utilities

CROSSBIN = $(NV_TOPDIR)/../arm-2009q3/bin/arm-none-linux-gnueabi-
CC     = $(CROSSBIN)gcc
CXX    = $(CROSSBIN)g++
LD     = $(CROSSBIN)gcc
AR     = $(CROSSBIN)ar
RANLIB = $(CROSSBIN)ranlib
STRIP  = $(CROSSBIN)strip

NV_CGDIR = $(NV_TOPDIR)/utils/Cg
HEXIFY   = $(NV_CGDIR)/hexify.pl
CGC      = $(NV_CGDIR)/$(NV_HOST_OSTYPE)/cgc -quiet
SHADER_FIX = $(NV_CGDIR)/$(NV_HOST_OSTYPE)/shaderfix
CGC_CG   = $(CGC)
CGC_GLSL = $(CGC) -ogles
CGC_CG_VTX   = $(CGC) -profile ar20vp
CGC_CG_FRG   = $(CGC) -profile ar20fp
CGC_GLSL_VTX = $(CGC_GLSL) -profile ar20vp
CGC_GLSL_FRG = $(CGC_GLSL) -profile ar20fp

# Rules for compiling shaders.
# C source files should #include an ASCII hex version of the shader binary
# in a .h file.  The following rules turn a shader program into a .h file.
%.h: %.cgbin
	$(HEXIFY) $< $@

%.cghex: %.cgbin
	$(HEXIFY) $< $@

%.cgbin: %.glslv
	$(CGC_GLSL_VTX) -o $*.tmp $<
	$(SHADER_FIX) -o $@ $*.tmp
	rm -rf $*.tmp

%.cgbin: %.glslf
	$(CGC_GLSL_FRG) -o $*.tmp $<
	$(SHADER_FIX) -o $@ $*.tmp
	rm -rf $*.tmp

%.cgbin: %.cgv
	$(CGC_CG_VTX) -o $*.tmp $<
	$(SHADER_FIX) -o $@ $*.tmp
	rm -rf $*.tmp

%.cgbin: %.cgf
	$(CGC_CG_FRG) -o $*.tmp $<
	$(SHADER_FIX) -o $@ $*.tmp
	rm -rf $*.tmp
