#include <KD/kd.h>
#include <GLES2/gl2.h>

#include "random.h"
#include "shaders.h"
#include "firefly.h"

// arrays holding firefly data
float *fPos;
float *fVel;
float *fColor;
float *fHsva;

static void hsva2rgba(float3 rgba, float4 hsva)
{
    float hue = hsva[0] - (float) kdFloorf((double) hsva[0]);
    float sat = clamp(hsva[1], 0.0f, 1.0f);
    float val = clamp(hsva[2], 0.0f, 1.0f);

    float hh = hue * 6.0f;
    float p,q,t;

    int i = (int)hh;

    hh -= i;

    p = val * (1 - sat);
    q = val * (1 - sat * hh);
    t = val * (1 - (sat * (1 - hh)));

    switch (i) {
    case 0: set_3(rgba, val, t, p); break;
    case 1: set_3(rgba, q, val, p); break;
    case 2: set_3(rgba, p, val, t); break;
    case 3: set_3(rgba, p, q, val); break;
    case 4: set_3(rgba, t, p, val); break;
    case 5: set_3(rgba, val, p, q); break;
    }
}

void Firefly_global_init(int num)
{
    fPos   = (float *)kdMalloc(sizeof(float) * num * 3);
    fVel   = (float *)kdMalloc(sizeof(float) * num * 3);
    fColor = (float *)kdMalloc(sizeof(float) * num * 3);
    fHsva  = (float *)kdMalloc(sizeof(float) * num * 4);
}

void Firefly_init(Firefly *o, int num)
{
    o->pos = fPos + (num * 3);
    o->vel = fVel + (num * 3);
    o->c = fColor + (num * 3);
    o->hsva = fHsva + (num * 4);

    set_3(o->pos, 0, 0, 3.0);
    set_3(o->vel, 0, 0, 0);
    set_3(o->c, 1, 1, 1);
    o->range = 8;
    set_4(o->hsva, (float)get_random(), 0.4f, 1.0f, 1.0f);
}

void Firefly_destroy(Firefly *o) {}

void Firefly_global_destroy(void)
{
    kdFree(fPos);
    kdFree(fVel);
    kdFree(fColor);
    kdFree(fHsva);
}

void Firefly_move(Firefly *o)
{
//    static unsigned int s=0;

    float r1 = ((float) get_random()) * 2.0f - 1.0f;
    float r2 = ((float) get_random()) * 2.0f - 1.0f;
    float r3 = ((float) get_random()) * 2.0f - 1.0f;

    float hs = 0.005f;
    float rs = 0.08f;
    float3 r = {rs * r1, rs * r2, rs * r3};

    float3 tmp;
    float3 home;

    o->hsva[0] += 0.01f;
    if (o->hsva[0] > 1.0) o->hsva[0] -= 1.0;
    hsva2rgba(o->c, o->hsva);

    //TODO:
    //srand(s++);

    home[0] = hs * -o->pos[0];
    home[1] = hs * -o->pos[1];
    home[2] = hs * (-o->pos[2]+o->range/2.0f);

    add_f3(tmp, o->vel, home);
    addi_f3(tmp, r);
    mult_f3f(o->vel, tmp, 0.97f);
    mult_f3f(tmp, o->vel, 0.5f);
    addi_f3(o->pos, tmp);

    if(o->pos[2] < 0.5) { o->pos[2] = 0.5; }
}


#define NUM (10)

void Firefly_draw(Firefly *o) {
    int i;
    float vertices[(NUM+1)*3];
    float colors  [(NUM+1)*4];
    float *v = vertices, *c = colors;

    for (i=0; i<NUM+1; i++) {
        *c++ = o->c[0];
        *c++ = o->c[1];
        *c++ = o->c[2];
        *c++ = 0.0;
    }
    colors[3] = 1.0;

    *v++ = o->pos[0];
    *v++ = o->pos[1];
    *v++ = o->pos[2];

    for (i=0; i<NUM; ++i) {
        *v++ = o->pos[0] + (((float) get_random())-0.5f) * 0.2f;
        *v++ = o->pos[1] + (((float) get_random())-0.5f) * 0.2f;
        *v++ = o->pos[2] + (((float) get_random())-0.5f) * 0.2f;
    }

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glDepthMask(GL_FALSE);

    glEnableVertexAttribArray(aloc_simplecolVertex);
    glEnableVertexAttribArray(aloc_simplecolColor);
    glVertexAttribPointer(aloc_simplecolVertex,
                          3, GL_FLOAT, GL_FALSE, 0, vertices);
    glVertexAttribPointer(aloc_simplecolColor,
                          4, GL_FLOAT, GL_FALSE, 0, colors);
    glDrawArrays(GL_TRIANGLE_FAN, 0, NUM+1);
    glDisableVertexAttribArray(aloc_simplecolVertex);
    glDisableVertexAttribArray(aloc_simplecolColor);

    glDisable(GL_BLEND);
    glDepthMask(GL_TRUE);
}
