#include <KD/kd.h>

#include "vbo.h"
#include "tree.h"
#include "buildtree.h"
#include "nvgl2demo_common.h"


// parameters to control the tree generation.
float tree_params[NUM_TREE_PARAMS] = {
   0.46f, // 0.07
   0.7f,
   degToRadF(60.0f),
   0.75f,
   2.0f, // 1.0
   0.1f, // 0.12
   0.5f, // 0.0
};

// minimum and maximum of the tree parameters.
float tree_params_min[NUM_TREE_PARAMS] = {
   0.0f,
   0.0f,
   0.0f,
   0.0f,
   0.5f,
   0.01f,
   0.0f,
};
float tree_params_max[NUM_TREE_PARAMS] = {
   1.0f,
   1.0f,
   degToRadF(180.0f),
   1.0f,
   5.0f,
   0.5f,
   1.0f,
};


// Variables specific to this module.
static GLboolean geometry_dirty = GL_FALSE;

static GLboolean is_VBO;

void Tree_newCharacter()
{
    BuildTree_newCharacter();
    geometry_dirty = GL_TRUE;
}


// construct (or reconstruct) the tree elements.
static void build(void)
{
    Branches_clear();
    Leaves_clear();

    BuildTree_generate();

    is_VBO = use_VBO;

    if (use_VBO) {
        // initialize the VBO area.
        if (VBO_setup(Leaves_sizeVBO() + Branches_sizeVBO() + Ground_sizeVBO()))
        {
            Leaves_buildVBO();
            Branches_buildVBO();
            Ground_buildVBO();

            is_VBO = GL_TRUE;
        } else {
            NvGl2DemoLogMessage("Unable to allocate VBO, falling back to non-VBO");
            is_VBO = GL_FALSE;
        }
    } else {
        is_VBO = GL_FALSE;
    }

    // Mark the dirty bit false.
    geometry_dirty = GL_FALSE;
}


void Tree_initialize(GLuint bark, GLuint leaf, GLuint leafb)
{
    VBO_init();

    // initialize the textures.
    Leaves_initialize(leaf, leafb, tree_params[TREE_PARAM_LEAF_SIZE]);
    Branches_initialize(bark);

    // this will cause the tree to be built at the first draw call.
    geometry_dirty = GL_TRUE;
}



void Tree_deinitialize()
{
    Leaves_deinitialize();
    Branches_deinitialize();
    if (use_VBO) {
        VBO_deinit();
    }
    Tree_newCharacter();
}



void Tree_draw() {
    if (geometry_dirty) {
        build();
    }
    Leaves_draw(is_VBO);
    Branches_draw(is_VBO);

    Ground_draw(is_VBO);
}


void Tree_setParam(int param, float val)
{
    tree_params[param] = val;
    geometry_dirty = GL_TRUE;
}

void Tree_toggleVBO()
{
    if (vbo_initialized) {
        use_VBO = !use_VBO;
        geometry_dirty = GL_TRUE;
    }
}
