#include <KD/kd.h>
#include <EGL/egl.h>

#include "vbo.h"
#include "nvgl2demo_common.h"

static int vboptr = 0;
static unsigned int vbosize = 0;
int vbo_initialized = 0;
int use_VBO = 0;

GLboolean VBO_init(void)
{
    vbo_initialized = 1;
    use_VBO = 1;
    return GL_TRUE;
}


void VBO_deinit()
{
    GLuint bufs[] = { VBO_NAME };

    if (vbo_initialized) {
        glDeleteBuffers(sizeof bufs / sizeof *bufs, bufs);
    }

    vbosize = vboptr = 0;
}


GLboolean VBO_setup(int size)
{
    int res;

    // clear out prior errors
    while (glGetError() != GL_NO_ERROR);

    VBO_deinit();
    glBindBuffer(GL_ARRAY_BUFFER, VBO_NAME);
    glBufferData(GL_ARRAY_BUFFER, size, KD_NULL, GL_DYNAMIC_DRAW);
    if ((res = glGetError()) == GL_NO_ERROR) {
        vbosize = size;
        return GL_TRUE;
    }
    else {
        do {
            // Print out the error
            NvGl2DemoLogMessage("Error: %x\n", res);
        } while ((res = glGetError()) != GL_NO_ERROR);
        return GL_FALSE;
    }
}


unsigned int VBO_alloc(int size)
{
    unsigned int ret = 0;
    size = VBO_align(size);
    if (((unsigned int) size) > vbosize) {
        NvGl2DemoLogMessage("VBO out of space");
    } else {
        ret = vboptr;
        vboptr += size;
        vbosize -= size;
    }
    return ret;
}


